from PySide6.QtWidgets import (
    QHBoxLayout, QVBoxLayout, QLabel, QWidget, QMenu, QSizePolicy, QLayout, QMessageBox
)
from PySide6.QtGui import Qt
from PySide6.QtCore import qDebug

from models import Member, MemberStatusEnum
from database.manager import DatabaseManager
from sqlalchemy import delete

STATUS_TO_COLOR_MAP = {
    MemberStatusEnum.active: "#3c702e",
    MemberStatusEnum.inactive: "#702525"
}


class MemberCard(QWidget):
    def __init__(self, member: Member):
        super().__init__()

        self.member = member

        self.setAttribute(Qt.WidgetAttribute.WA_Hover, True)
        self.setAttribute(Qt.WidgetAttribute.WA_StyledBackground, True)

        # Set initial stylesheet with hover behavior
        self.setStyleSheet("""
            MemberCard:hover {
                background-color: palette(highlight);
            }
        """)

        # Layout setup
        layout = QHBoxLayout(self)
        layout.setSizeConstraint(QLayout.SizeConstraint.SetMinimumSize)
        self.setSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)

        layout.setContentsMargins(10, 10, 10, 10)
        layout.setSpacing(10)

        # Left-side content
        left_side = QVBoxLayout()
        layout.addLayout(left_side)
        name_label = QLabel(f"{member.first_name} {member.last_name}")
        name_label.setStyleSheet("font-size: 20px; font-weight: bold;")
        email_label = QLabel(f"Email: {member.email}")
        phone_label = QLabel(f"Phone: {member.phone or 'Not Available'}")
        left_side.addWidget(name_label)
        left_side.addWidget(email_label)
        left_side.addWidget(phone_label)

        # Right-side content
        right_side = QVBoxLayout()
        layout.addLayout(right_side)

        status_label = QLabel(str(member.status.value.capitalize()))
        status_label.setStyleSheet(f"color: {STATUS_TO_COLOR_MAP[member.status]}; font-size: 20px; font-weight: bold;")
        status_label.setAlignment(Qt.AlignmentFlag.AlignRight)

        register_date_label = QLabel(f"Registered: {member.register_date}")
        register_date_label.setAlignment(Qt.AlignmentFlag.AlignRight)

        right_side.addWidget(status_label)
        right_side.addWidget(register_date_label)

        self.setLayout(layout)

    def mousePressEvent(self, event):
        if event.button() == Qt.MouseButton.LeftButton:
            self.contextMenuEvent(event)
        else:
            super().mousePressEvent(event)

    def contextMenuEvent(self, event):
        context_menu = QMenu(self)

        action_edit_member = context_menu.addAction("Edit Member")
        action_deactivate_member = context_menu.addAction("Deactivate Member")
        action_activate_member = context_menu.addAction("Activate Member")
        context_menu.addSeparator()
        delete_member_action = context_menu.addAction("Delete Member")
        delete_member_action.triggered.connect(self.delete_member)

        if self.member.status == MemberStatusEnum.active:
            action_activate_member.setVisible(False)
        elif self.member.status == MemberStatusEnum.inactive:
            action_deactivate_member.setVisible(False)

        action = context_menu.exec_(self.mapToGlobal(event.pos()))

        if action == action_edit_member:
            print("Edit Member selected")  # Implement editor logic here
        elif action == action_deactivate_member:
            self.update_member_status(MemberStatusEnum.inactive)
        elif action == action_activate_member:
            self.update_member_status(MemberStatusEnum.active)

    def delete_member(self):
        self.make_sure()
        # if not self.make_sure():
        #     return
        
        # with DatabaseManager.get_session() as session:
        #     try:
        #         stmt = delete(Member).where(Member.id == self.member.id)
        #         session.execute(stmt)
        #         session.commit()
        #         self.setVisible(False)
        #     except Exception as e:
        #         session.rollback()
        #         print(e)

    def update_member_status(self, new_status):
        with DatabaseManager.get_session() as session:
            try:
                member = session.query(Member).filter(Member.id == self.member.id).one_or_none()
                if member:
                    member.status = new_status
                    session.commit()
                    QMessageBox.information(self, "Status Updated", f"Member status updated to {new_status.value.capitalize()}.")
                    self.member.status = new_status
                    self.update_status_label()
                else:
                    QMessageBox.critical(self, "Error", "The member you requested could not be found.", QMessageBox.StandardButton.Ok)
            except Exception as e:
                session.rollback()
                print(e)

    def update_status_label(self):
        self.findChild(QLabel, self.member.status.value).setStyleSheet(
            f"color: {STATUS_TO_COLOR_MAP[self.member.status]}; font-size: 20px; font-weight: bold;"
        )

    def make_sure(self) -> bool:
        are_you_sure_box = QMessageBox()
        are_you_sure_box.setIcon(QMessageBox.Question)
        are_you_sure_box.setWindowTitle("Are you sure?")
        are_you_sure_box.setText(f"Are you sure you want to delete {self.member.first_name} {self.member.last_name}?")
        are_you_sure_box.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
        are_you_sure_box.setDefaultButton(QMessageBox.No)

        response = are_you_sure_box.exec()
        return response == QMessageBox.Yes
