from PySide6.QtWidgets import (
    QWidget, QVBoxLayout, QHBoxLayout, QLabel, QLineEdit, QScrollArea,
    QPushButton, QMessageBox, QVBoxLayout
)
from PySide6.QtCore import Qt

from .member_card import MemberCard
from models import Member
from database.manager import DatabaseManager

from ui.editor import MemberEditor


class MemberList(QWidget):
    def __init__(self):
        super().__init__()

        # Central widget and layout
        main_layout = QVBoxLayout(self)

        # Title label
        title_label = QLabel("Member Dashboard", self)
        title_label.setAlignment(Qt.AlignCenter)
        title_label.setStyleSheet(
            "font-size: 20px; font-weight: bold; color: #0078D4;")
        main_layout.addWidget(title_label)

        # Search bar
        self.search_input = QLineEdit()
        self.search_input.setPlaceholderText("Search members...")
        self.search_input.textChanged.connect(self.filter_members)
        main_layout.addWidget(self.search_input)

        # Scrollable area for cards
        self.scroll_area = QScrollArea()
        self.scroll_area.setWidgetResizable(True)

        # Container widget for the scroll area
        self.scroll_widget = QWidget()
        self.scroll_layout = QVBoxLayout(self.scroll_widget)
        self.scroll_layout.setSpacing(5)  # Set gap between individual cards
        self.scroll_layout.setContentsMargins(0, 0, 0, 0)  # Remove default spacing

        # Align the cards to the top
        self.scroll_layout.setAlignment(Qt.AlignTop)
        self.members = []
        self.member_cards = []
        self.redraw_cards()

        self.scroll_widget.setLayout(self.scroll_layout)
        self.scroll_area.setWidget(self.scroll_widget)
        main_layout.addWidget(self.scroll_area)

        # Buttons for actions
        button_layout = QHBoxLayout()
        register_member_button = QPushButton("Add New Member")
        register_member_button.clicked.connect(self.register_member)
        button_layout.addWidget(register_member_button)

        delete_member_button = QPushButton("Delete Member")
        delete_member_button.clicked.connect(self.delete_member)
        button_layout.addWidget(delete_member_button)

        main_layout.addLayout(button_layout)

    def filter_members(self, text):
        """Filter the cards based on the search input."""
        for card, member in zip(self.member_cards, self.members):
            name_contains_text = text.lower() in member.name.lower()
            id_contains_text = text.lower() in str(member.id)

            card.setVisible(name_contains_text or id_contains_text)

    def register_member(self):
        MemberEditor().exec()

    def delete_member(self):
        QMessageBox.information(self, "Delete Member",
                                "Open dialog to delete a member.")

    def clear_layout(self, layout):
        while layout.count():
            item = layout.takeAt(0)
            widget = item.widget()
            if widget is not None:
                widget.deleteLater()
            else:
                sub_layout = item.layout()
                if sub_layout is not None:
                    self.clear_layout(sub_layout)
            del item

    def redraw_cards(self):
        self.clear_layout(self.scroll_layout)
        self.member_cards = []

        self.members = self.fetch_members_from_db()

        for member in self.members:
            card = MemberCard(member)

            self.scroll_layout.addWidget(card)
            self.member_cards.append(card)

    def fetch_members_from_db(self):
        """Fetch all members from the database."""
        try:
            with DatabaseManager.get_session() as session:
                members = session.query(Member).all()
                return members
        except Exception as e:
            QMessageBox.critical(self, "Database Error",
                                 f"Failed to fetch members: {e}")
            return []
